# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gs_aws_to_gcp_workload_identity']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.24.40,<2.0.0',
 'botocore>=1.27.40,<2.0.0',
 'google-auth>=2.9.1,<3.0.0',
 'jwt>=1.3.1,<2.0.0',
 'requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'gs-aws-to-gcp-workload-identity',
    'version': '0.1.1',
    'description': 'Helper for obtaining GCP Tokens via AWS-to-GCP Workload Identity Federation',
    'long_description': '# AWS to GCP Workload Identity Federation\n\nIf you have workloads running in AWS and want to access GCP services without exporting static GCP service account credentials, this module can leverage workload identity federation to dynamically exchange credentials from an IAM Role in AWS for either an OAuth2 Access Token or Identity Token for a given GCP Service Account.\n\n## Getting Started\n\n### AWS Setup\n\nSet some shell variables for AWS related items for easier setup:\n\n```bash\nexport AWS_ACCOUNT_ID="123456789012"\nexport AWS_ROLE_NAME="demo-wif-iam-role"\n# If you are running this on an EC2 instance or Lambda with a role attached, the ARN is just the role ARN\nexport AWS_ROLE_ARN="arn:aws:iam::${AWS_ACCOUNT_ID}:role/${AWS_ROLE_NAME}"\n# If you are assuming a role via some other means, use this ARN format instead\n# export AWS_ROLE_ARN="arn:aws:sts::${AWS_ACCOUNT_ID}:assumed-role/${AWS_ROLE_NAME}"\n```\n\nNote that the `AWS_ROLE_NAME` can be the name of an existing role (for example, attached to an EC2 instance or Lambda) or you can create a new one.  It needs no specific policies attached on the AWS side, but you can if that role needs to use AWS services as well.\n\nHere\'s an example role that can be attached to EC2 or Lambda with no AWS related policies attached:\n\n```bash\naws iam get-role --role-name demo-wif-iam-role\n{\n    "Role": {\n        "Path": "/",\n        "RoleName": "demo-wif-iam-role",\n        "RoleId": "AROA3K4K4NKZMWMTAAMLD",\n        "Arn": "arn:aws:iam::123456789012:role/demo-wif-iam-role",\n        "CreateDate": "2022-07-08T15:38:50+00:00",\n        "AssumeRolePolicyDocument": {\n            "Version": "2012-10-17",\n            "Statement": [\n                {\n                    "Effect": "Allow",\n                    "Principal": {\n                        "Service": [\n                            "ec2.amazonaws.com",\n                            "lambda.amazonaws.com"\n                        ]\n                    },\n                    "Action": "sts:AssumeRole"\n                }\n            ]\n        },\n        "Description": "Allows EC2 instances to call AWS services on your behalf.",\n        "MaxSessionDuration": 3600,\n        "Tags": [\n            {\n                "Key": "env",\n                "Value": "dev"\n            }\n        ],\n        "RoleLastUsed": {\n            "LastUsedDate": "2022-07-29T14:40:18+00:00",\n            "Region": "us-east-1"\n        }\n    }\n}\n```\n\n### GCP Setup\n\nSet some shell variables for GCP related items:\n\n```bash\nexport GCP_PROJECT_ID="my-project-id"\nexport GCP_WORKLOAD_POOL_ID="aws-to-gcp-pool"\nexport GCP_WORKLOAD_PROVIDER_ID="aws-to-gcp-provider"\nexport GCP_SA_NAME="my-wif-sa"\n```\n\nAuthenticate with `gcloud` as an `roles/owner` on the target GCP project, and set the current project:\n\n```bash\ngcloud config set project "${GCP_PROJECT_ID}"\n```\n\nObtain the GCP Project Number:\n\n```bash\nexport GCP_PROJECT_NUMBER="$(gcloud projects describe ${GCP_PROJECT_ID} --format=\'value(projectNumber)\')"\n```\n\nEnable the necessary services in the project:\n\n```bash\ngcloud services enable sts.googleapis.com iamcredentials.googleapis.com\n```\n\nCreate the Workload Identity Pool:\n\n```bash\ngcloud iam workload-identity-pools create "${GCP_WORKLOAD_POOL_ID}" \\\n  --location="global" \\\n  --display-name="${GCP_WORKLOAD_POOL_ID}" \\\n  --description="${GCP_WORKLOAD_POOL_ID}"\n```\n\nCreate an `aws` Provider in the Workload Identity Pool:\n\n```bash\ngcloud iam workload-identity-pools providers create-aws "${GCP_WORKLOAD_PROVIDER_ID}" \\\n  --account-id="${AWS_ACCOUNT_ID}" \\\n  --location="global" \\\n  --workload-identity-pool="${GCP_WORKLOAD_POOL_ID}"\n```\n\nCreate a dedicated GCP service account that the AWS IAM Role will be connected to:\n\n```bash\nexport GCP_SA_EMAIL="${GCP_SA_NAME}@${GCP_PROJECT_ID}.iam.gserviceaccount.com"\ngcloud iam service-accounts create "${GCP_SA_NAME}" \\\n  --display-name="${GCP_SA_NAME}" \\\n  --description="WIF from AWS to ${GCP_SA_NAME}"\n```\n\nOr reference an existing GCP service account:\n\n```bash\nexport GCP_SA_EMAIL="my-existing-sa@my-project-id.iam.gserviceaccount.com"\n```\n\nGrant permissions directly on the service account so the AWS Role ARN can exchange STS for its GCP token(s):\n\n```bash\ngcloud iam service-accounts add-iam-policy-binding "${GCP_SA_EMAIL}" \\\n  --member "principalSet://iam.googleapis.com/projects/${GCP_PROJECT_NUMBER}/locations/global/workloadIdentityPools/${GCP_WORKLOAD_POOL_ID}/attribute.aws_role/${AWS_ROLE_ARN}" \\\n  --role roles/iam.workloadIdentityUser\n```\n\nGrant permissions as usual for the GCP service account to access services in GCP.  Next, list the workload identity pools/providers to get the full `gcp_workload_provider_path` to use in the module:\n\n```bash\nfor pool in "$(gcloud iam workload-identity-pools list --location global --format=\'value(name)\')"; do\n  gcloud iam workload-identity-pools providers list --location global --workload-identity-pool="${pool}" --format=\'value(name)\';\ndone\n\nprojects/4556456456456/locations/global/workloadIdentityPools/aws-to-gcp-pool/providers/aws-to-gcp-provider\n```\n\n## Usage\n\n```python\n#!/usr/bin/env python3\n\nfrom gs_aws_to_gcp_workload_identity.main import AwsToGcpTokenService\n\naws_to_gcp_token_service = AwsToGcpTokenService(\n    gcp_workload_provider_path="<insert full workload identity path>",\n    gcp_service_account_email="<insert GCP service account email>",\n    aws_assume_role_arn=None # Insert a role ARN to explicitly assume first if needed\n)\n\n# Most GCP services take the OAuth2 access token as the Bearer Token\naccess_token, expiry_utc_epoch = aws_to_gcp_token_service.get_oauth_token()\n\n# Certain services like GCP Functions require Identity Tokens for Authenticated\n# invocations and need the function URL in the token\'s `audience` field\nfunction_url="https://us-central1-my-project-id.cloudfunctions.net/hello_world"\nid_token, expiry_utc_epoch = aws_to_gcp_token_service.get_identity_token(audience=function_url)\n\n# Call GCP services here\n```\n\n## Samples\n\nSee [SAMPLES.md](./SAMPLES.md) for working examples.\n\n## FAQ\n\n- **What about performance and rate limits?** - Performing this token exchange requires several calls to AWS\' STS APIs and GCP\'s STS/IAM APIs, so this module caches the credentials after the first exchange and refreshes them only if they are expired or under 300s from expiring.  This should help reduce the API quote/rate limit usage.  Running hundreds of Lambdas at once is likely to hit some of these, for example.\n\n',
    'author': 'Ghost Security',
    'author_email': 'info@ghost.security',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ghostsecurity/gs-aws-to-gcp-workload-identity',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
